from abc import ABCMeta, abstractmethod
from typing import List, Dict

from project_utils.web.django.swagger import Params, ParamsTypes, Query
from project_utils.web.django.view import SwaggerType
from ._types import REQUEST_QUERY, REQUEST_BODY, RESPONSE, REQUEST_FORM


class BaseDocumentType(metaclass=ABCMeta):
    path: REQUEST_QUERY
    query: REQUEST_QUERY
    form: REQUEST_FORM
    response_page: RESPONSE
    response_get: RESPONSE
    response_post: RESPONSE
    response_put: RESPONSE
    response_delete: RESPONSE = SwaggerType().response({
        "delete_count": Params(ParamsTypes.integer, title="删除数量").params()
    })
    base_query: REQUEST_QUERY = [
        Query("pagenum", require=True, description="分页页码").params(),
        Query("pagesize", require=True, description="分页大小").params(),
        Query("startTime", description="开始时间").params(),
        Query("endTime", description="截止时间").params()
    ]

    @classmethod
    @abstractmethod
    def body(cls, require: List[str]) -> REQUEST_BODY:
        ...

    @classmethod
    @abstractmethod
    def page(cls, properties: Dict) -> RESPONSE:
        ...



from enum import Enum

from typing import Optional, Dict, Union, Any, List

from drf_yasg.openapi import Schema, Parameter, IN_FORM

from ._base import SwaggerParamsBase
from ._types import SwaggerParamsEnum


class SwaggerFormParams(SwaggerParamsBase):
    name: str
    type: Union[SwaggerParamsEnum, Any]
    require: bool
    description: Optional[str]
    position: str

    def __init__(self, name: str, type: Union[SwaggerParamsEnum, Any], require: bool = False,
                 description: Optional[str] = None):
        self.name = name
        self.type = type
        self.require = require
        self.description = description
        self.position = IN_FORM

    def params(self, properties: Optional[Dict[str, Schema]] = None, items: Schema = None) -> Union[Schema, Parameter]:
        return Parameter(name=self.name,
                         in_=self.position,
                         description=self.description,
                         required=self.require,
                         type=self.type)



import traceback

from asgiref.sync import sync_to_async
from django.db.models import Model, Manager, CharField, BooleanField

from project_utils.exception import ModelException
from ._model_type import String, Bool
from ._id_model import IDModel
from ._base_meta import BaseMeta


class BaseModel(Model):
    class Meta(BaseMeta):
        abstract: bool = True

    table_id: int = 1000
    objects: Manager = Manager()

    id: String = CharField(max_length=34, verbose_name="ID", blank=True, primary_key=True,
                           default=IDModel.create_id(table_id))
    is_delete: Bool = BooleanField(verbose_name="删除标记", blank=True, default=False)

    @classmethod
    async def async_filter(cls, *args: ..., **kwargs: ...):
        return await sync_to_async(cls.objects.filter)(*args, **kwargs)

    @classmethod
    def get_fields(cls):
        return [field.name for field in cls._meta.get_fields()]

    @classmethod
    async def async_get_field(cls):
        return [field.name for field in cls._meta.get_fields()]

    @classmethod
    def get_one(cls, *args, **kwargs):
        try:
            return cls.objects.get(*args, **kwargs)
        except Exception as e:
            raise ModelException(str(e), error_detail=traceback.format_exc())

    @classmethod
    async def async_get_one(cls, *args, **kwargs):
        try:
            return await cls.objects.aget(*args, **kwargs)
        except Exception as e:
            raise ModelException(str(e), traceback.format_exc())



from typing import Union

from django.db.models import *

from datetime import datetime

Int = Union[int, IntegerField]
Bool = Union[bool, BooleanField]
Byte = Union[bytes, BinaryField]
Float = Union[float, FloatField]
String = Union[str, CharField]

Long = Union[int, BigIntegerField]
File = FileField
Url = Union[str, URLField]
Email = Union[str, EmailField]
DateTime = Union[datetime, DateTimeField]
Json = Union[str, JSONField]
Text = Union[str, TextField]



from typing import Callable, Any, Dict, List
from asgiref.sync import sync_to_async

from project_utils.exception import ViewException


class DjangoUtils:
    @staticmethod
    async def to_async_fun(fun: Callable, *args, **kwargs) -> Any:
        return await sync_to_async(fun)(*args, **kwargs)

    @staticmethod
    async def check(params: Dict[str, Any], field_list: List[str]):
        for field in field_list:
            assert field in params and params.get(field) is not None, ViewException(403,
                                                                                    f"Field name's {field} is not in request field list or name's {field} value is null!")



import asyncio
import traceback

from asgiref.sync import async_to_sync
from django.http.request import HttpRequest
from django.http.response import HttpResponse

from project_utils.exception import ViewException, ModelException
from project_utils.web.django.conf import DjangoConfig
from project_utils.web.django.request import WebRequest
from project_utils.web.django.response import WebResponse

from .api_util import BaseAPIUtil

from .._types import BASE_API_TYPE, VIEW_TYPE, API_VIEW_HANDLER_TYPE


def base_api(config: DjangoConfig, method: str = "get") -> BASE_API_TYPE:
    def decorator(fun: VIEW_TYPE) -> API_VIEW_HANDLER_TYPE:
        def handler(request: HttpRequest, *args: ..., **kwargs: ...) -> HttpResponse:
            try:
                api_util: BaseAPIUtil = BaseAPIUtil(config=config, request=request, method=method)
            except ViewException as e:
                config.printf.error(e.error_detail)
                config.printf.warning(e.error_summary)
                return WebResponse(status=e.status, error=e).to_response()
            except ModelException as e:
                config.printf.error(e.error_detail)
                config.printf.warning(e.error_summary)
                return WebResponse(status=404, error=e).to_response()
            except Exception as e:
                view_exception: ViewException = ViewException(405, str(e), traceback.format_exc())
                config.printf.error(view_exception.error_detail)
                config.printf.warning(view_exception.error_summary)
                return WebResponse(status=view_exception.status, error=view_exception).to_response()
            try:
                web_request: WebRequest = api_util.request_init(config)
            except ViewException as e:
                config.printf.error(e.error_detail)
                config.printf.warning(e.error_summary)
                return WebResponse(status=e.status, error=e).to_response()
            except Exception as e:
                view_exception: ViewException = ViewException(403, str(e), traceback.format_exc())
                config.printf.error(view_exception.error_detail)
                config.printf.warning(view_exception.error_summary)
                return WebResponse(status=view_exception.status, error=view_exception).to_response()
            try:
                if asyncio.iscoroutinefunction(fun):
                    web_response: WebResponse = async_to_sync(fun)(web_request, *args, **kwargs)
                else:
                    web_response: WebResponse = fun(web_request, *args, **kwargs)
            except ViewException as e:
                config.printf.error(e.error_detail)
                config.printf.warning(e.error_summary)
                return WebResponse(status=e.status, error=e).to_response()
            except Exception as e:
                view_exception: ViewException = ViewException(410, str(e), traceback.format_exc())
                config.printf.error(view_exception.error_detail)
                config.printf.warning(view_exception.error_summary)
                return WebResponse(status=view_exception.status, error=view_exception).to_response()
            return web_response.to_response()

        return handler

    return decorator



import time

from datetime import datetime, timedelta


def datetime_to_str(datetime: datetime = datetime.now(), format="%Y-%m-%d %H:%M:%S") -> str:
    return datetime.strftime(format)


def timestamp_to_str(timestamp: float = time.time(), format: str = "%Y-%m-%d %H:%M:%S") -> str:
    return datetime_to_str(datetime.fromtimestamp(timestamp), format)


def datetime_to_timestamp(datetime: datetime = datetime.now()) -> float:
    return datetime.timestamp()


def str_to_datetime(date_str: str, format: str = "%Y-%m-%d %H:%M:%S") -> datetime:
    return datetime.strptime(date_str, format)


def str_to_timestamp(date_str: str, format: str = "%Y-%m-%d %H:%M:%S") -> float:
    date_attr: datetime = str_to_datetime(date_str, format)
    return date_attr.timestamp()


def timestamp_to_datetime(timestamp: float) -> datetime:
    return datetime.fromtimestamp(timestamp)


def compute_year(year: int = 0, date: datetime = datetime.now()) -> datetime:
    year: int = date.year - year
    month: int = date.month
    day: int = date.day
    return datetime(year, month, day)


def compute_month(month: int = 0, date: datetime = datetime.now()) -> datetime:
    year: int = date.year
    month = date.month - month
    day = date.day
    return datetime(year, month, day)


def compute_week(week: int = 0, date: datetime = datetime.now()) -> datetime:
    return date + timedelta(weeks=week)


def compute_day(day: int = 0, date: datetime = datetime.now()) -> datetime:
    return date + timedelta(days=day)


def compute_hour(hour: int = 0, date: datetime = datetime.now()) -> datetime:
    return date + timedelta(hours=hour)


def compute_minute(minute: int = 0, date: datetime = datetime.now()) -> datetime:
    return date + timedelta(minutes=minute)


def compute_second(second: int = 0, date: datetime = datetime.now()) -> datetime:
    return date + timedelta(seconds=second)


if __name__ == '__main__':
    print(len(timestamp_to_str(is_date=True)))



from copy import deepcopy
from typing import Optional, Union, Any, Dict

from drf_yasg.openapi import Schema

from ._base import SwaggerParamsBase
from ._types import SwaggerParamsEnum


class SwaggerParamsResponse:
    class SwaggerParamsResponseInner(SwaggerParamsBase):
        type: str = SwaggerParamsEnum.object
        title: Optional[str]
        description: Optional[str]

        def __init__(self, type: Union[SwaggerParamsEnum, Any], title: Optional = None,
                     description: Optional[str] = None):
            self.type = type.value
            self.title = title
            self.description = description

        def params(self, properties: Optional[Dict[str, Schema]] = None, items: Schema = None) -> Union[Schema]:
            return Schema(type=self.type, title=self.title, description=self.description, properties=properties,
                          items=items)

    inner = SwaggerParamsResponseInner
    base: Dict[str, Schema] = {
        "errno": inner(SwaggerParamsEnum.string, title="响应码", description="响应码").params(),
        "errmsg": inner(SwaggerParamsEnum.string, title="响应描述", description="响应描述").params()
    }

    def success(self, properties: Optional[Dict[str, Schema]] = None, items: Optional[Schema] = None) -> Schema:
        base: Dict[str, Schema] = deepcopy(self.base)
        if properties:
            base.update({
                "data": self.inner(SwaggerParamsEnum.object, description="响应数据").params(properties=properties)
            })
        if items:
            base.update({
                "data": self.inner(SwaggerParamsEnum.array, description="响应数据").params(items=items)
            })
        return Schema(type=SwaggerParamsEnum.object.value, properties=base)

    def error(self) -> Schema:
        base: Dict[str, Schema] = deepcopy(self.base)
        base.update({
            "error": self.inner(SwaggerParamsEnum.string, "错误信息").params()
        })
        return Schema(type=SwaggerParamsEnum.object.value, properties=base)



import traceback

from abc import ABCMeta
from typing import Optional, List, Dict, Any, Union, Tuple

from django.db.models import Q
from django.db.models.query import QuerySet

from rest_framework.serializers import ReturnDict, ReturnList

from drf_yasg.openapi import Schema, Parameter
from drf_yasg.utils import swagger_auto_schema

from project_utils.exception import ViewException, DjangoWebException, ModelException
from project_utils.web.django.conf import DjangoConfig
from project_utils.web.django.model import BaseModel
from project_utils.web.django.serializer import BaseSerializer
from project_utils.web.django.pagination import BasePagination
from project_utils.web.django.request import WebRequest
from project_utils.web.django.response import WebResponse

from .._types import PARAMS_TYPE
from .._rest_api.api import RestAPIView
from project_utils.web.django.view._swagger_base import SwaggerTypeBase


class BaseRestAPIView(RestAPIView, metaclass=ABCMeta):
    require:List[str]
    pop_field:List[str]
    config: DjangoConfig
    model: BaseModel
    serializer: BaseSerializer
    page_class: BasePagination

    # 删除标记，False表示逻辑删除，True表示物理删除
    is_delete: bool = False
    # 过滤连接条件
    filter_condition: str = "OR"
    # 过滤符号
    filter_symbol: str = "contains"

    page_instance: BasePagination
    is_page: bool = True

    def dispatch(self, request, *args, **kwargs):
        try:
            return super().dispatch(request, *args, **kwargs)
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(e.status, error=e).to_rest_response()
        except ModelException as e:
            self.show_exception(e)
            return WebResponse(404, error=e).to_rest_response()
        except Exception as e:
            exception: ViewException = self.create_exception(410, e)
            self.show_exception(exception)
            return WebResponse(exception.status, error=exception).to_rest_response()

    def create_exception(self, status: int, e: Exception) -> ViewException:
        return ViewException(status, str(e), traceback.format_exc())

    def show_exception(self, e: DjangoWebException):
        self.config.printf.error(e.error_detail)
        self.config.printf.warning(e.error_summary)

    def __get_filter_field(self, params: PARAMS_TYPE) -> Dict[str, Any]:
        field: List[str] = self.model.get_fields()
        params_keys: List[str] = list(params.keys())
        filter_field: List[str] = list(set(field) & set(params_keys))
        filter_mapper: Dict[str, Any] = {key: params.get(key)[0] for key in filter_field}
        return filter_mapper

    def __filter(self, params: PARAMS_TYPE) -> QuerySet[BaseModel]:
        filter_field: Dict[str, Any] = self.__get_filter_field(params)
        instance: QuerySet[BaseModel] = self.model.objects.filter(is_delete=0)
        q: Q = Q()
        q.connector = self.filter_symbol
        for key, val in filter_field.items():
            if key == "is_delete":
                q.children.append((key, val))
            else:
                q.children.append((f"{key}__{self.filter_symbol}", val))
        return instance.filter(q)

    def check(self, params: PARAMS_TYPE, require_fields: List[str]):
        for field in require_fields:
            assert field in params and params.get(field) is not None, ViewException(403,
                                                                                    f"Field name's {field} is not in request field list or name's {field} value is null!")

    def before_select(self, request: WebRequest, instance: Union[QuerySet[BaseModel], BaseModel], many: bool) -> \
            Union[QuerySet[BaseModel], BaseModel]:
        return instance

    def __before_select(self, request: WebRequest, params: PARAMS_TYPE, many: bool, select_id: Optional[str]) -> \
            Union[QuerySet[BaseModel], BaseModel]:
        if many:
            instance: QuerySet[BaseModel] = self.__filter(params)
        else:
            instance: BaseModel = self.model.get_one(id=select_id)
        try:
            instance: Union[QuerySet[BaseModel], BaseModel] = self.before_select(request, instance, many)
            if not many and instance.is_delete:
                raise Exception("not find!")
        except Exception as e:
            raise self.create_exception(400, e)
        if many and self.is_page:
            self.page_instance = self.page_class()
            instance: QuerySet[BaseModel] = self.page_instance.paginate_queryset(instance, request.request, view=self)
        return instance

    @swagger_auto_schema(auto_schema=None)
    def get(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is get\nselect id is {id}")
        params: PARAMS_TYPE = request.params
        many: bool = id is None
        instance: QuerySet[BaseModel] = self.__before_select(request, params, many, id)
        serializer: BaseSerializer = self.serializer(instance=instance, many=many)
        data: Union[ReturnDict, ReturnList] = serializer.data.copy()
        data = self.__selected(request, data, many)
        request.config.printf.success(f"View success!\nPath is {request.path}\nMethod is {request.method}")
        return WebResponse(data=data)

    def __selected(self, request: WebRequest, data: Union[ReturnList, ReturnDict], many: bool) -> Union[
        ReturnDict, ReturnList]:
        try:
            data: Union[ReturnList, ReturnDict] = self.selected(request, data, many)
        except Exception as e:
            raise self.create_exception(410, e)
        if many and self.is_page:
            data: Dict[str, Any] = self.page_instance.get_paginated_response(data)
        return data

    def selected(self, request: WebRequest, data: Union[ReturnDict, ReturnList], many: bool) -> Union[
        ReturnDict, ReturnList]:
        return data

    def before_create(self, request: WebRequest, params: Dict[str, Any]) -> Dict[str, Any]:
        return params

    def __before_create(self, request: WebRequest, params: PARAMS_TYPE) -> Dict[str, Any]:
        try:
            return self.before_create(request, params)
        except Exception as e:
            raise self.create_exception(400, e)

    @swagger_auto_schema(auto_schema=None)
    def post(self, request: WebRequest) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is post")
        params: PARAMS_TYPE = request.params
        data: Dict[str, Any] = self.__before_create(request, params)
        serializer: BaseSerializer = self.serializer(data=data)
        try:
            serializer.is_valid(raise_exception=True)
        except Exception as e:
            raise self.create_exception(401, e)
        serializer.save()
        result: ReturnDict = self.__created(request, params, serializer)
        request.config.printf.success(f"View success!\nPath is {request.path}\nMethod is {request.method}")
        return WebResponse(data=result)

    def __created(self, request: WebRequest, params: PARAMS_TYPE, serializer: BaseSerializer) -> ReturnDict:
        try:
            return self.created(request, params, serializer.data.copy())
        except Exception as e:
            raise self.create_exception(410, e)

    def created(self, request: WebRequest, params: Dict[str, Any], data: ReturnDict) -> ReturnDict:
        return data

    def before_update(self, request: WebRequest, params: Dict[str, Any], instance: BaseModel) -> Tuple[
        Dict[str, Any], BaseModel]:
        return params, instance

    def __before_update(self, request: WebRequest, params: PARAMS_TYPE, instance: BaseModel) -> Tuple[
        Dict[str, Any], BaseModel]:
        try:
            return self.before_update(request, params, instance)
        except Exception as e:
            raise self.create_exception(400, e)

    @swagger_auto_schema(auto_schema=None)
    def put(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is put\nupdate id is {id}")
        assert id, ViewException(404, "update id value isn't null!")
        try:
            instance: BaseModel = self.model.get_one(id=id)
        except ModelException as e:
            raise self.create_exception(404, e)
        params: PARAMS_TYPE = request.params
        try:
            params, instance = self.__before_update(request, params, instance)
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=401, error=e)
        serializer: BaseSerializer = self.serializer(instance=instance, data=params)
        try:
            serializer.is_valid(raise_exception=True)
        except Exception as e:
            _e: ViewException = self.create_exception(e)
            self.show_exception(_e)
            return WebResponse(status=402, error=_e)
        serializer.save()
        try:
            result: ReturnDict = self.__updated(request, serializer, params)
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=410, error=e)
        request.config.printf.success(f"View success!\nPath is {request.path}\nMethod is {request.method}")
        return WebResponse(data=result)

    def __updated(self, request: WebRequest, serializer: BaseSerializer, params: PARAMS_TYPE) -> ReturnDict:
        try:
            return self.updated(request, serializer.data.copy(), params)
        except Exception as e:
            raise self.create_exception(410, e)

    def updated(self, request: WebRequest, data: ReturnDict, params: Dict[str, Any]) -> ReturnDict:
        return data

    def before_delete(self, request: WebRequest, params: Dict[str, Any], many: bool,
                      instance: Optional[BaseModel] = None) -> Union[QuerySet[BaseModel], BaseModel]:
        return instance

    def __before_delete(self, request: WebRequest, params: PARAMS_TYPE, many: bool,
                        delete_id: Optional[str] = None) -> Union[QuerySet[BaseModel], BaseModel]:
        instance: Optional[BaseModel] = None
        if not many:
            try:
                instance = self.model.get_one(id=delete_id)
            except ModelException as e:
                raise self.create_exception(404, e)
        try:
            return self.before_delete(request, params, many, instance)
        except Exception as e:
            raise self.create_exception(400, e)

    @swagger_auto_schema(auto_schema=None)
    def delete(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is delete\ndelete id is {id}")
        many: bool = id is None
        params: PARAMS_TYPE = request.params
        delete: Union[QuerySet[BaseModel], BaseModel] = self.__before_delete(request, params, many, id)
        serializer: BaseSerializer = self.serializer(instance=delete, many=many)
        data: Union[ReturnList, ReturnDict] = serializer.data.copy()
        delete_count: int = 0
        if many:
            for delete_item in delete:
                delete_count += 1
                if self.is_delete:
                    delete_item.delete()
                else:
                    delete_item.is_delete = True
                    delete_item.save()
        else:
            delete_count += 1
            if self.is_delete:
                delete.delete()
            else:
                delete.is_delete = True
                delete.save()
        self.__deleted(request, params, data)
        request.config.printf.success(f"View success!\nPath is {request.path}\nMethod is {request.method}")
        return WebResponse(data={"delete_count": delete_count})

    def __deleted(self, request: WebRequest, params: PARAMS_TYPE, data: Union[ReturnList, ReturnDict]):
        try:
            self.deleted(request, params, data)
        except Exception as e:
            raise self.create_exception(410, e)

    def deleted(self, request: WebRequest, params: Dict[str, Any], data: Union[ReturnList, ReturnDict]):
        pass

    def pop(self, data: ReturnDict, pop_field: Optional[List[str]] = None) -> ReturnDict:
        for field in pop_field:
            data.pop(field)
        return data



import traceback

from abc import ABCMeta
from typing import Optional, Dict, Any, Union, Tuple, List

from django.db.models.query import QuerySet, Q

from rest_framework.serializers import ReturnDict, ReturnList

from drf_yasg.openapi import Schema, Parameter
from drf_yasg.utils import swagger_auto_schema

from project_utils.exception import ViewException, ModelException
from project_utils.web.django.utils import DjangoUtils
from project_utils.web.django.conf import DjangoConfig
from project_utils.web.django.model import BaseModel
from project_utils.web.django.serializer import BaseSerializer
from project_utils.web.django.pagination import BasePagination
from project_utils.web.django.request import WebRequest
from project_utils.web.django.response import WebResponse

from .._rest_api import AsyncRestApi
from .._types import PARAMS_TYPE

from project_utils.web.django.view._swagger_base import SwaggerTypeBase


class AsyncBaseAPIView(AsyncRestApi, metaclass=ABCMeta):
    require: List[str]
    pop_field: List[str]
    config: DjangoConfig
    model: BaseModel
    serializer: BaseSerializer
    page_class: BasePagination

    # 删除标记，False表示逻辑删除，True表示物理删除
    is_delete: bool = False
    # 过滤连接条件
    filter_condition: str = "OR"
    # 过滤符号
    filter_symbol: str = "contains"

    page_instance: BasePagination
    is_page: bool = True

    async def dispatch(self, request, *args, **kwargs):
        try:
            return await super().dispatch(request, *args, **kwargs)
        except ViewException as e:
            await self.show_exception(e)
            response = WebResponse(e.status, error=e)
            self.response = self.finalize_response(request, response, *args, **kwargs)
            return self.response
        except Exception as e:
            exception: ViewException = await self.create_exception(410, e)
            await self.show_exception(exception)
            response = WebResponse(exception.status, error=exception)
            self.response = self.finalize_response(request, response, *args, **kwargs)
            return self.response

    async def __get_filter_field(self, params: PARAMS_TYPE) -> Dict[str, Any]:
        field: List[str] = await self.model.async_get_field()
        params_keys: List[str] = list(params.keys())
        filter_field: List[str] = list(set(field) & set(params_keys))
        filter_mapper: Dict[str, Any] = {key: params.get(key)[0] for key in filter_field}
        return filter_mapper

    async def create_exception(self, status: int, e: Exception) -> ViewException:
        return ViewException(status, str(e), traceback.format_exc())

    async def __filter(self, params: PARAMS_TYPE) -> QuerySet[BaseModel]:
        filter_field: Dict[str, Any] = await self.__get_filter_field(params)
        instance: QuerySet[BaseModel] = await self.model.async_filter(is_delete=0)
        q: Q = Q()
        q.connector = self.filter_symbol
        for key, val in filter_field.items():
            if key == "is_delete":
                q.children.append((key, val))
            else:
                q.children.append((f"{key}__{self.filter_symbol}", val))
        return await DjangoUtils.to_async_fun(instance.filter, q)

    async def before_select(self, request: WebRequest, instance: Union[QuerySet[BaseModel], BaseModel], many: bool) -> \
            Union[QuerySet[BaseModel], BaseModel]:
        return instance

    async def __before_select(self, request: WebRequest, params: PARAMS_TYPE, many: bool, select_id: Optional[str]) -> \
            Union[QuerySet[BaseModel], BaseModel]:
        if many:
            instance: QuerySet[BaseModel] = await self.__filter(params)
        else:
            instance: BaseModel = await self.model.async_get_one(id=select_id)
            if instance.is_delete:
                raise Exception("not find!")
        try:
            instance: Union[QuerySet[BaseModel], BaseModel] = await self.before_select(request, instance, many)
        except Exception as e:
            raise await self.create_exception(status=400, e=e)
        if many and self.is_page:
            self.page_instance = self.page_class()
            instance = await self.page_instance.async_paginate_queryset(instance, request.request, self)
        return instance

    @swagger_auto_schema(auto_schema=None)
    async def get(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is get\nselect id is {id}")
        params: PARAMS_TYPE = request.params
        many: bool = id is None
        instance: Union[QuerySet[BaseModel], BaseModel] = await self.__before_select(request, params, many, id)
        serializer: BaseSerializer = self.serializer(instance=instance, many=many)
        data: Union[ReturnList, ReturnDict] = serializer.data.copy()
        result: Union[ReturnList, ReturnDict] = await self.__selected(request, data, many)
        request.config.printf.success(f"View success!\nPath is {request.path}\nMethod is {request.method}")
        return WebResponse(data=result)

    async def __selected(self, request: WebRequest, data: Union[ReturnList, ReturnDict], many: bool) -> Union[
        ReturnDict, ReturnList]:
        try:
            data: Union[ReturnList, ReturnDict] = await self.selected(request, data, many)
        except Exception as e:
            raise await self.create_exception(410, e)
        if many and self.is_page:
            data: Dict[str, Any] = await self.page_instance.get_paginated_data(data)
        return data

    async def selected(self, request: WebRequest, data: Union[ReturnDict, ReturnList], many: bool) -> Union[
        ReturnDict, ReturnList]:
        return data

    async def before_create(self, request: WebRequest, params: Dict[str, Any]) -> Dict[str, Any]:
        return params

    async def __before_create(self, request: WebRequest, params: PARAMS_TYPE) -> Dict[str, Any]:
        try:
            return await self.before_create(request, params)
        except Exception as e:
            raise await self.create_exception(400, e)

    @swagger_auto_schema(auto_schema=None)
    async def post(self, request: WebRequest) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is post")
        params: PARAMS_TYPE = request.params
        data: Dict[str, Any] = await self.__before_create(request, params)
        serializer: BaseSerializer = self.serializer(data=data)
        try:
            await DjangoUtils.to_async_fun(serializer.is_valid, raise_exception=True)
        except Exception as e:
            raise await self.create_exception(401, e)
        await serializer.async_save()
        result: ReturnDict = await self.__created(request, params, serializer)
        request.config.printf.success(f"View success!\nPath is {request.path}\nMethod is {request.method}")
        return WebResponse(data=result)

    async def __created(self, request: WebRequest, params: PARAMS_TYPE, serializer: BaseSerializer) -> ReturnDict:
        try:
            return await self.created(request, params, serializer.data.copy())
        except Exception as e:
            raise await self.create_exception(410, e)

    async def created(self, request: WebRequest, params: Dict[str, Any], data: ReturnDict) -> ReturnDict:
        return data

    async def before_update(self, request: WebRequest, params: Dict[str, Any], instance: BaseModel) -> Tuple[
        Dict[str, Any], BaseModel]:
        return params, instance

    async def __before_update(self, request: WebRequest, params: PARAMS_TYPE, instance: BaseModel) -> Tuple[
        Dict[str, Any], BaseModel]:
        try:
            return await self.before_update(request, params, instance)
        except Exception as e:
            raise await self.create_exception(400, e)

    @swagger_auto_schema(auto_schema=None)
    async def put(self, request: WebRequest, id: Optional[str] = None):
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is put\nupdate id is {id}")
        assert id, ViewException(error_summary="update id value isn't null!")
        try:
            instance: BaseModel = await self.model.async_get_one(id=id)
        except ModelException as e:
            raise await self.create_exception(404, e)
        params: PARAMS_TYPE = request.params
        params, instance = await self.__before_update(request, params, instance)
        serializer: BaseSerializer = self.serializer(instance=instance, data=params)
        try:
            await DjangoUtils.to_async_fun(serializer.is_valid, raise_exception=True)
        except Exception as e:
            raise await self.create_exception(403, e)
        await serializer.async_save()
        result: ReturnDict = await self.__updated(request, serializer, params)
        request.config.printf.success(f"View success!\nPath is {request.path}\nMethod is {request.method}")
        return WebResponse(data=result)

    async def __updated(self, request: WebRequest, serializer: BaseSerializer, params: PARAMS_TYPE) -> ReturnDict:
        try:
            return await self.updated(request, serializer.data.copy(), params)
        except Exception as e:
            raise await self.create_exception(410, e)

    async def updated(self, request: WebRequest, data: ReturnDict, params: Dict[str, Any]) -> ReturnDict:
        return data

    async def before_delete(self, request: WebRequest, params: Dict[str, Any], many: bool,
                            instance: Optional[BaseModel] = None) -> Union[QuerySet[BaseModel], BaseModel]:
        return instance

    async def __before_delete(self, request: WebRequest, params: PARAMS_TYPE, many: bool,
                              delete_id: Optional[str] = None) -> Union[QuerySet[BaseModel], BaseModel]:
        instance: Optional[BaseModel] = None
        if not many:
            try:
                instance: BaseModel = await self.model.async_get_one(id=delete_id)
            except ModelException as e:
                raise e

        try:
            return await self.before_delete(request, params, many, instance)
        except Exception as e:
            raise await self.create_exception(400, e)

    @swagger_auto_schema(auto_schema=None)
    async def delete(self, request: WebRequest, id: Optional[str] = None):
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is delete\ndelete id is {id}")
        params: PARAMS_TYPE = request.params
        many: bool = id is None
        instance: Union[QuerySet[BaseModel], BaseModel] = await self.__before_delete(request, params, many,
                                                                                     delete_id=id)
        serializer: BaseSerializer = self.serializer(instance=instance, many=many)
        data: Union[ReturnList, ReturnDict] = serializer.data.copy()
        delete_count: int = 0
        if many:
            for item in instance:
                delete_count += 1
                if self.is_delete:
                    await item.adelete()
                else:
                    item.is_delete = True
                    await item.asave()
        else:
            delete_count += 1
            if self.is_delete:
                await instance.adelete()
            else:
                instance.is_delete = True
                await instance.asave()
        await self.__deleted(request, params, data)
        request.config.printf.success(f"View success!\nPath is {request.path}\nMethod is {request.method}")
        return WebResponse(data={"delete_count": delete_count})

    async def __deleted(self, request: WebRequest, params: PARAMS_TYPE, data: Union[ReturnList, ReturnDict]):
        try:
            await self.deleted(request, params, data)
        except Exception as e:
            raise await self.create_exception(410, e)

    async def deleted(self, request: WebRequest, params: Dict[str, Any], data: Union[ReturnList, ReturnDict]):
        pass

    async def check(self, params: Dict[str, Any], field_list: List[str]):
        for field in field_list:
            assert field in params and params.get(field) is not None, ViewException(403,
                                                                                    f"Field name's {field} is not in request field list or name's {field} value is null!")

    async def pop(self, data: ReturnDict, pop_field: List[str]) -> ReturnDict:
        for field in pop_field:
            data.pop(field)
        return data



import asyncio

from asgiref.sync import sync_to_async
from rest_framework import exceptions, status
from rest_framework.request import Request
from rest_framework.views import APIView


class AsyncAPIView(APIView):
    """
    参考:
        https://github.com/encode/django-rest-framework/issues/7260
    Provides async view compatible support for DRF Views and ViewSets.

    This must be the first inherited class.
    """

    @classmethod
    def as_view(cls, **initkwargs):
        """Make Django process the view as an async view."""
        view = super().as_view(**initkwargs)

        async def async_view(*args, **kwargs):
            # wait for the `dispatch` method
            return await view(*args, **kwargs)

        async_view.cls = cls
        async_view.initkwargs = initkwargs
        async_view.csrf_exempt = True
        return async_view

    async def get_exception_handler_context(self):
        # 返回异常上下文
        return await sync_to_async(super().get_exception_handler_context)()

    async def get_exception_handler(self):
        # 不论异常函数是不是协程
        return super().get_exception_handler()

    async def handle_exception(self, exc):
        """
        Handle any exception that occurs, by returning an appropriate response,
        or re-raising the error.
        """
        if isinstance(exc, (exceptions.NotAuthenticated, exceptions.AuthenticationFailed)):
            # WWW-Authenticate header for 401 responses, else coerce to 403
            auth_header = await sync_to_async(self.get_authenticate_header)(self.request)

            if auth_header:
                exc.auth_header = auth_header
            else:
                exc.status_code = status.HTTP_403_FORBIDDEN

        exception_handler = await self.get_exception_handler()

        context = await self.get_exception_handler_context()

        # 如果自定义异常函数不是协程
        if not asyncio.iscoroutinefunction(exception_handler):
            response = await sync_to_async(exception_handler)(exc, context)
        else:
            response = await exception_handler(exc, context)

        if response is None:
            await sync_to_async(self.raise_uncaught_exception)(exc)

        response.exception = True
        return response

    async def initialize_request(self, request, *args, **kwargs):
        """
        Returns the initial request object.
        """
        parser_context = self.get_parser_context(request)

        return Request(
            request,
            parsers=self.get_parsers(),
            authenticators=self.get_authenticators(),
            negotiator=self.get_content_negotiator(),
            parser_context=parser_context,
        )

    async def dispatch(self, request, *args, **kwargs):
        """Add async support."""
        self.args = args
        self.kwargs = kwargs
        request = await self.initialize_request(request, *args, **kwargs)
        self.request = request
        self.headers = self.default_response_headers

        try:
            await sync_to_async(self.initial)(request, *args, **kwargs)

            if request.method.lower() in self.http_method_names:
                handler = getattr(self, request.method.lower(), self.http_method_not_allowed)
            else:
                handler = self.http_method_not_allowed

            # accept both async and sync handlers
            # built-in handlers are sync handlers

            # if not asyncio.iscoroutinefunction(handler):
            #     handler = sync_to_async(handler)
            response = await handler(request, *args, **kwargs)


        except Exception as exc:
            response = await self.handle_exception(exc)

        self.response = self.finalize_response(request, response, *args, **kwargs)
        return self.response



import traceback

from abc import ABCMeta, abstractmethod
from typing import Optional

from rest_framework.views import APIView
from rest_framework.request import Request

from project_utils.exception import ViewException, DjangoWebException
from project_utils.web.django.conf import DjangoConfig
from project_utils.web.django import WebRequest, WebResponse

from .api_utils import BaseRestAPI


class RestAPIView(APIView, metaclass=ABCMeta):
    config: DjangoConfig

    def initialize_request(self, request, *args, **kwargs):
        request: Request = super().initialize_request(request, *args, **kwargs)
        api_util: BaseRestAPI = BaseRestAPI(request, method=request.method, config=self.config)
        web_request: WebRequest = api_util.request_init(self.config)
        return web_request

    def show_exception(self, e: DjangoWebException):
        self.config.printf.error(e.error_detail)
        self.config.printf.warning(e.error_summary)

    def create_exception(self, status: int, e: Exception) -> ViewException:
        return ViewException(status=status, error_summary=str(e), error_detail=traceback.format_exc())

    def initial(self, request, *args, **kwargs):
        self.request = request.request
        return super().initial(request.request)

    def finalize_response(self, request, response, *args, **kwargs):
        request: WebRequest = request
        response: WebResponse = response
        return super().finalize_response(request, response.to_rest_response())

    @abstractmethod
    def get(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        ...

    @abstractmethod
    def post(self, request: WebRequest) -> WebResponse:
        ...

    @abstractmethod
    def put(self, request: WebRequest, id: Optional[str]) -> WebResponse:
        ...

    @abstractmethod
    def delete(self, request: WebRequest, id: Optional[str]) -> WebResponse:
        ...



from ._document import BaseDocumentType
from ._types import REQUEST_BODY, REQUEST_QUERY, REQUEST_FORM, RESPONSE

BaseDocument = BaseDocumentType

__all__ = [
    "BaseDocument",
    "REQUEST_QUERY",
    "REQUEST_BODY",
    "REQUEST_FORM",
    "RESPONSE"
]



import traceback

from typing import Any, Optional

from pymysql import connect
from pymysql.connections import Connection
from pymysql.cursors import DictCursor
from dbutils.pooled_db import PooledDB
from dbutils.steady_db import SteadyDBConnection

from project_utils.exception import MysqlException

from ._collection import MysqlCollection
from ._result import MysqlResult


class MysqlPool(MysqlCollection):
    use_pool: bool
    connection: Connection
    pool: Optional[PooledDB] = None

    def __init__(self, creator: Any, use_pool: bool = True, *args, **kwargs):
        self.use_pool = use_pool
        super().__init__(max_connections=10, min_cached=5, max_cached=10, max_shared=10)
        if self.use_pool:
            if self.pool is None:
                self.pool = PooledDB(
                    creator,
                    self.min_cached,
                    self.max_cached,
                    self.max_shared,
                    self.max_connections,
                    self.blocking,
                    setsession=self.set_session,
                    ping=self.ping,
                    *args, **kwargs
                )
        else:
            self.connection = connect(*args, **kwargs)

    @property
    def conn(self):
        if self.use_pool:
            return self.pool.connection()
        else:
            return self.connection

    def running(self, sql: str, data: Any = None, many: bool = False) -> MysqlResult:
        if self.use_pool:
            conn: SteadyDBConnection = self.pool.connection()
        else:
            conn: Connection = self.connection
        cursor: DictCursor = conn.cursor(DictCursor)
        result: MysqlResult = self.execute(conn, cursor, sql, data, many)
        return result

    def add_task(self, conn: SteadyDBConnection, sql: str, data: Any = None, many: bool = False) -> MysqlResult:
        cursor: DictCursor = conn.cursor()
        result: MysqlResult = self.async_execute(cursor, sql, data, many)
        return result

    def commit(self, conn: SteadyDBConnection):
        try:
            conn.commit()
        except Exception as e:
            conn.rollback()
            raise MysqlException(str(e), traceback.format_exc())



from rest_framework.request import Request

from .._base import Base
from .._types import PARAMS_TYPE


class BaseRestAPI(Base):
    request: Request

    def parse_params(self) -> PARAMS_TYPE:
        method: str = self.method.lower()

        params: PARAMS_TYPE = {}
        if method in ("get", "delete"):
            params.update(self.request.query_params)
        elif method in ("post", "put", "delete"):
            params.update(self.request.data)
        return params



from typing import Union, Dict, Any, Optional

from django.http.request import HttpRequest
from django.contrib.sessions.backends.db import SessionStore

from rest_framework.request import Request

from project_utils.web.django import DjangoConfig


class WebRequest:
    _config: DjangoConfig
    _request: Union[HttpRequest, Request]
    _user: Optional[Dict[str, str]]
    _params: Optional[Dict[str, Any]]

    def __init__(self, request: Union[HttpRequest, Request], config: DjangoConfig):
        self._request = request
        self._config = config

    @property
    def user(self) -> Dict[str, str]:
        return self._user

    @user.setter
    def user(self, value: Dict[str, str]):
        self._user = value

    @property
    def params(self):
        return self._params

    @params.setter
    def params(self, value):
        self._params = value

    @property
    def session(self) -> SessionStore:
        return self._request.session

    @property
    def path(self) -> str:
        return self._request.path

    @property
    def meta(self) -> Dict[str, str]:
        return self._request.META

    @property
    def request(self) -> Union[Request, HttpRequest]:
        return self._request

    @property
    def config(self) -> DjangoConfig:
        return self._config

    @property
    def method(self) -> str:
        return self._request.method



from .api import RestAPIView
from .async_api import AsyncRestApi

rest_api = rest_api_view = RestAPIView
async_rest_api = async_rest_api_view = AsyncRestApi

__all__ = [
    "rest_api",
    "rest_api_view",
    "RestAPIView",
    "async_rest_api_view",
    "async_rest_api",
    "AsyncRestApi"
]



from abc import ABCMeta, abstractmethod

from typing import Optional, Dict, Union

from drf_yasg.openapi import Schema


class SwaggerParamsBase(metaclass=ABCMeta):
    @abstractmethod
    def params(self, properties: Optional[Dict[str, Schema]] = None, items: Schema = None) -> Union[Schema]:
        pass



from typing import Dict, Union

from project_utils.exception import ConfigException
from .base_config import BaseConfig


class RedisConfig(BaseConfig):
    password: str
    db: int

    def __init__(self, host: str, password: str, db: str, port: str = "6379"):
        assert db.isdigit(), ConfigException("params db type required integer!")
        super().__init__(host=host, port=port)
        self.password = password
        self.db = int(db)

    def to_dict(self) -> Dict[str, Union[str, int]]:
        result: Dict[str, Union[str, int]] = super().to_dict()
        result.update({
            "password": self.password,
            "db": self.db
        })
        return result

    def to_url(self) -> str:
        return f"redis://:{self.password}@{self.host}:{self.port}/{self.db}"



from typing import Callable, Any

from drf_yasg.openapi import Info
from drf_yasg.views import get_schema_view

from .config import SwaggerConfig

swagger_view_callable: Callable[[SwaggerConfig], Any] = lambda config: get_schema_view(Info(**config.swagger.to_dict()))



from project_utils.web.django.conf import DjangoConfig

from ._swagger import Swagger


class SwaggerConfig(DjangoConfig):
    __swagger: Swagger

    def config_init(self, base_url: str) -> None:
        super().config_init(base_url)

    def django_setting_init(self):
        super().django_setting_init()
        self.add_app("drf_yasg")
        self.__swagger = Swagger(**self.parser["SWAGGER"])
        
    @property
    def swagger(self) -> Swagger:
        return self.__swagger



from typing import Dict, Union

from .base_config import BaseConfig


class MysqlConfig(BaseConfig):
    user: str
    password: str
    database: str

    def __init__(self, host: str, user: str, password: str, database: str, port: str = "3306"):
        super().__init__(host=host, port=port)
        self.user = user
        self.password = password
        self.database = database

    def to_dict(self) -> Dict[str, Union[str, int]]:
        result: Dict[str, Union[str, int]] = super().to_dict()
        result.update({
            "user": self.user,
            "password": self.password,
            "database": self.database
        })
        return result



from abc import ABCMeta, abstractmethod
from typing import Optional

from rest_framework.request import Request

from project_utils.exception import DjangoWebException, ViewException
from project_utils.web.django import WebRequest, WebResponse
from project_utils.web.django.conf import DjangoConfig

from .api_utils import BaseRestAPI
from .._async_view import AsyncAPIView


class AsyncRestApi(AsyncAPIView, metaclass=ABCMeta):
    config: DjangoConfig

    async def initialize_request(self, request, *args, **kwargs):
        request: Request = await super().initialize_request(request, *args, **kwargs)
        self.headers = self.default_response_headers
        api_util: BaseRestAPI = BaseRestAPI(request, method=request.method, config=self.config)
        web_request: WebRequest = api_util.request_init(self.config)
        return web_request

    async def show_exception(self, e: DjangoWebException):
        self.config.printf.error(e.error_detail)
        self.config.printf.warning(e.error_summary)

    async def create_exception(self, status: int, e: Exception) -> ViewException:
        return ViewException(status=status, error_summary=str(e), error_detail=traceback.format_exc())

    def initial(self, request, *args, **kwargs):
        self.request = request.request
        return super().initial(request.request)

    def finalize_response(self, request, response, *args, **kwargs):
        if type(response) == WebResponse:
            response = response.to_rest_response()
        return super().finalize_response(request, response, *args, **kwargs)

    @abstractmethod
    async def get(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        ...

    @abstractmethod
    async def post(self, request: WebRequest) -> WebResponse:
        ...

    @abstractmethod
    async def put(self, request: WebRequest, id: Optional[str]) -> WebResponse:
        ...

    @abstractmethod
    async def delete(self, request: WebRequest, id: Optional[str]) -> WebResponse:
        ...



from django.conf import settings
from celery import Celery


def create_app(project_name: str, settings_path: str = "django.config:settings") -> Celery:
    app: Celery = Celery(project_name, backend=settings.CELERY_BROKER_URL)
    app.config_from_object(settings_path, namespace='CELERY')
    app.autodiscover_tasks()
    return app



from typing import Any, Optional, Dict

from drf_yasg.openapi import Contact, License


class Swagger:
    title: str
    default_version: str
    description: str
    terms_of_service: str
    contact: Contact
    license: License

    __instance__: Any = None

    def __new__(cls, *args, **kwargs):
        if cls.__instance__ is None:
            cls.__instance__ = object.__new__(cls)
        return cls.__instance__

    def __init__(
            self,
            title: str,
            email: str,
            license: str = "BSD License",
            default_version: str = "v1",
            description: Optional[str] = None,
            terms_of_service: Optional[str] = None
    ):
        self.title = title
        self.default_version = default_version
        self.description = description
        self.terms_of_service = terms_of_service
        self.contact = Contact(email=email)
        self.license = License(name=license)

    def to_dict(self) -> Dict[str, Any]:
        return {
            "title": self.title,
            "default_version": self.default_version,
            "description": self.description,
            "terms_of_service": self.terms_of_service,
            "contact": self.contact,
            "license": self.license
        }



from abc import ABCMeta
from typing import Dict, Union, Any

from project_utils.exception import ConfigException


class BaseConfig(metaclass=ABCMeta):
    __instance__: Any = None
    host: str
    port: int

    def __new__(cls, *args, **kwargs):
        if cls.__instance__ is None:
            cls.__instance__ = object.__new__(cls)
        return cls.__instance__

    def __init__(self, port: str, host: str = "0.0.0.0"):
        assert port.isdigit(), ConfigException("params port type required integer!")
        self.host = host
        self.port = int(port)

    def to_dict(self) -> Dict[str, Union[str, int]]:
        return {
            "host": self.host,
            "port": self.port
        }



from typing import Optional, List, Dict, Any, Union

from drf_yasg.openapi import Schema

from ._base import SwaggerParamsBase
from ._types import SwaggerParamsEnum


class SwaggerParams(SwaggerParamsBase):
    type: Union[SwaggerParamsEnum, Any]
    title: Optional[str]
    require: Optional[List[str]]
    description: Optional[str]
    properties: Optional[Dict[str, Schema]]

    def __init__(self, type: Union[SwaggerParamsEnum, Any], title: Optional[str] = None,
                 require: Optional[List[str]] = None,
                 description: Optional[str] = None):
        """

        :rtype: object
        """
        self.type = type
        self.title = title
        self.require = require
        self.description = description

    def params(self, properties: Optional[Dict[str, Schema]] = None, items: Schema = None) -> Union[Schema]:
        return Schema(
            title=self.title,
            type=self.type.value,
            required=self.require,
            description=self.description,
            properties=properties,
            items=items
        )


if __name__ == '__main__':
    from drf_yasg.openapi import TYPE_OBJECT



from enum import Enum
from typing import List, Dict, Union

from drf_yasg.openapi import Parameter, Schema

REQUEST_QUERY = Union[List[Parameter], Enum]
REQUEST_BODY = Union[Schema, Enum]
REQUEST_FORM = Union[List[Parameter], Enum]
RESPONSE = Union[Dict[str, Schema], Enum]